//
// Shige-Fuji Project
// SwiftUI Archives
//
// Copyright (c) Watanabe-Denki, Inc.
//   https://wdkk.co.jp/
//
// This software is released under the MIT License.
//   https://opensource.org/licenses/mit-license.php
//

import UIKit
import RealityKit

class CustomARView: ARView {
    // サーフェスシェーダの名前
    let surfaceShaderName: String = "textureSurfaceShader"
    
    // アンカー
    var firstAnchor: AnchorEntity = AnchorEntity()

    required init(frame frameRect: CGRect) {
        super.init(frame: frameRect)
        createScene()
    }
    
    @MainActor required dynamic init?(coder decoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    func prepareAnchor() {
        // 平面検出無しのアンカー。アプリ起動時のデバイスの位置を元に設置
        firstAnchor.position = simd_make_float3(0, -0.5, -1) // アンカーの位置は、デバイス初期位置から0.5m下、1m奥（ARKitでは1m = 1.0）
        self.scene.anchors.append(firstAnchor)
    }
    
    func makeCustomMaterial(textureResorceName texResrcName: String = "") -> CustomMaterial {
        // デバイスの参照を取得
        guard let maybeDevice = MTLCreateSystemDefaultDevice() else {
            fatalError("Error creating default metal device.")
        }
        // ライブラリの参照を取得
        guard let maybeLibrary = maybeDevice.makeDefaultLibrary() else {
            fatalError("Error creating default metal library")
        }
        // サーフェスシェーダの参照を取得
        let surface_shader = CustomMaterial.SurfaceShader(named: self.surfaceShaderName, in: maybeLibrary)
        // カスタムマテリアルを作成（クリアコート使用）
        var custom_material = try! CustomMaterial(surfaceShader: surface_shader, lightingModel: .clearcoat)
        
        // テクスチャの指定が無い場合はここで処理を抜ける
        if texResrcName == "" { return custom_material }
        
        // テクスチャを扱いたい場合
        if let resorce = try? TextureResource.load(named: texResrcName) {
            print("テクスチャリソース名:\(texResrcName)")
            // テクスチャの読み込み
            let texture = CustomMaterial.Texture(resorce)
            // カスタムマテリアルにテクスチャを設定
            custom_material.custom.texture = .init(texture) 
        }
        // 作成したマテリアルを返す
        return custom_material
    }
    
    func createScene() {
        // アンカーの準備
        prepareAnchor()
     
        // カスタムマテリアル作成
        let custom_material = makeCustomMaterial(textureResorceName: "LilyIcon.png")
        
        // 板ポリの表示位置
        let plane_transform = Transform(translation: simd_make_float3(0.0, 0.2, -0.1))

        // 板ポリ(PlaneMesh)を作成
        let plane_texture = makePlane(customMaterial: custom_material,
                                     transform: plane_transform,
                                     width: 0.5, height: 0.5)
        // 板ポリをアンカーに追加
        self.firstAnchor.addChild(plane_texture)
    }
    
    func makePlane(customMaterial: CustomMaterial, transform :Transform, width wid: Float = 0.25, height hgt: Float = 0.25) -> ModelEntity {
        let plane = ModelEntity(mesh: .generatePlane(width: wid, height: hgt))
        plane.model?.materials = [customMaterial]
        plane.transform = transform
        return plane
    }
}

